import sys
import logging
import socket
import cgi

import pytest

from webob.request import Request
from webob.response import Response
from webob.compat import url_open
from webob.compat import bytes_
from webob.compat import reraise
from webob.compat import Queue
from webob.compat import Empty

log = logging.getLogger(__name__)

@pytest.mark.usefixtures("serve")
def test_request_reading(serve):
    """
        Test actual request/response cycle in the presence of Request.copy()
        and other methods that can potentially hang.
    """
    with serve(_test_app_req_reading) as server:
        for key in _test_ops_req_read:
            resp = url_open(server.url + key, timeout=3)
            assert resp.read() == b"ok"

def _test_app_req_reading(env, sr):
    req = Request(env)
    log.debug('starting test operation: %s', req.path_info)
    test_op = _test_ops_req_read[req.path_info]
    test_op(req)
    log.debug('done')
    r = Response("ok")
    return r(env, sr)


_test_ops_req_read = {
    '/copy': lambda req: req.copy(),
    '/read-all': lambda req: req.body_file.read(),
    '/read-0': lambda req: req.body_file.read(0),
    '/make-seekable': lambda req: req.make_body_seekable()
}

@pytest.mark.usefixtures("serve")
def test_interrupted_request(serve):
    with serve(_test_app_req_interrupt) as server:
        for path in _test_ops_req_interrupt:
            _send_interrupted_req(server, path)
            try:
                res = _global_res.get(timeout=1)
            except Empty:
                raise AssertionError("Error during test %s", path)
            if res is not None:
                print("Error during test:", path)
                reraise(res)


_global_res = Queue()


def _test_app_req_interrupt(env, sr):
    target_cl = 100000
    try:
        req = Request(env)
        cl = req.content_length
        if cl != target_cl:
            raise AssertionError(
                'request.content_length is %s instead of %s' % (cl, target_cl))
        op = _test_ops_req_interrupt[req.path_info]
        log.info("Running test: %s", req.path_info)
        with pytest.raises(IOError):
            op(req)
    except:
        _global_res.put(sys.exc_info())
    else:
        _global_res.put(None)
        sr('200 OK', [])
        return []

def _req_int_cgi(req):
    assert req.body_file.read(0) == b''
    cgi.FieldStorage(
        fp=req.body_file,
        environ=req.environ,
    )

def _req_int_readline(req):
    try:
        assert req.body_file.readline() == b'a=b\n'
    except IOError:
        # too early to detect disconnect
        raise AssertionError("False disconnect alert")
    req.body_file.readline()


_test_ops_req_interrupt = {
    '/copy': lambda req: req.copy(),
    '/read-body': lambda req: req.body,
    '/read-post': lambda req: req.POST,
    '/read-all': lambda req: req.body_file.read(),
    '/read-too-much': lambda req: req.body_file.read(1 << 22),
    '/readline': _req_int_readline,
    '/readlines': lambda req: req.body_file.readlines(),
    '/read-cgi': _req_int_cgi,
    '/make-seekable': lambda req: req.make_body_seekable()
}


def _send_interrupted_req(server, path='/'):
    sock = socket.socket()
    sock.connect(('localhost', server.server_port))
    f = sock.makefile('wb')
    f.write(bytes_(_interrupted_req % path))
    f.flush()
    f.close()
    sock.close()


_interrupted_req = (
    "POST %s HTTP/1.0\r\n"
    "content-type: application/x-www-form-urlencoded\r\n"
    "content-length: 100000\r\n"
    "\r\n"
)
_interrupted_req += 'a=b\nz=' + 'x' * 10000



import pytest

import datetime
import calendar
from email.utils import formatdate
from webob import datetime_utils

def test_UTC():
    """Test missing function in _UTC"""
    x = datetime_utils.UTC
    assert x.tzname(datetime.datetime.now()) == 'UTC'
    assert x.dst(datetime.datetime.now()) == datetime.timedelta(0)
    assert x.utcoffset(datetime.datetime.now()) == datetime.timedelta(0)
    assert repr(x) == 'UTC'

def test_parse_date():
    """Testing datetime_utils.parse_date.
    We need to verify the following scenarios:
        * a nil submitted value
        * a submitted value that cannot be parse into a date
        * a valid RFC2822 date with and without timezone
    """

    ret = datetime_utils.parse_date(None)
    assert ret is None, ("We passed a None value to parse_date. We should get"
                         " None but instead we got %s" % ret)

    ret = datetime_utils.parse_date('Hi There')
    assert ret is None, ("We passed an invalid value to parse_date. We should"
                         " get None but instead we got %s" % ret)

    ret = datetime_utils.parse_date(1)
    assert ret is None, ("We passed an invalid value to parse_date. We should"
                         " get None but instead we got %s" % ret)

    ret = datetime_utils.parse_date('\xc3')
    assert ret is None, ("We passed an invalid value to parse_date. We should"
                         " get None but instead we got %s" % ret)

    ret = datetime_utils.parse_date('Mon, 20 Nov 1995 19:12:08 -0500')
    assert ret == datetime.datetime(
        1995, 11, 21, 0, 12, 8, tzinfo=datetime_utils.UTC)

    ret = datetime_utils.parse_date('Mon, 20 Nov 1995 19:12:08')

    assert ret == datetime.datetime(
        1995, 11, 20, 19, 12, 8, tzinfo=datetime_utils.UTC)

    ret = datetime_utils.parse_date(Uncooperative())
    assert ret is None

class Uncooperative(object):
    def __str__(self):
        raise NotImplementedError

def test_serialize_date():
    """Testing datetime_utils.serialize_date
    We need to verify the following scenarios:
        * on py3, passing an binary date, return the same date but str
        * on py2, passing an unicode date, return the same date but str
        * passing a timedelta, return now plus the delta
        * passing an invalid object, should raise ValueError
    """
    from webob.compat import text_
    ret = datetime_utils.serialize_date('Mon, 20 Nov 1995 19:12:08 GMT')
    assert isinstance(ret, str)
    assert ret == 'Mon, 20 Nov 1995 19:12:08 GMT'
    ret = datetime_utils.serialize_date(text_('Mon, 20 Nov 1995 19:12:08 GMT'))
    assert isinstance(ret, str)
    assert ret == 'Mon, 20 Nov 1995 19:12:08 GMT'
    dt = formatdate(
        calendar.timegm(
            (datetime.datetime.now() + datetime.timedelta(1)).timetuple()),
        usegmt=True)
    assert dt == datetime_utils.serialize_date(datetime.timedelta(1))
    with pytest.raises(ValueError):
        datetime_utils.serialize_date(None)

def test_parse_date_delta():
    """Testing datetime_utils.parse_date_delta
    We need to verify the following scenarios:
        * passing a nil value, should return nil
        * passing a value that fails the conversion to int, should call
          parse_date
    """
    assert datetime_utils.parse_date_delta(None) is None, ('Passing none value,'
                                                           'should return None')
    ret = datetime_utils.parse_date_delta('Mon, 20 Nov 1995 19:12:08 -0500')
    assert ret == datetime.datetime(
        1995, 11, 21, 0, 12, 8, tzinfo=datetime_utils.UTC)
    WHEN = datetime.datetime(2011, 3, 16, 10, 10, 37, tzinfo=datetime_utils.UTC)
    with _NowRestorer(WHEN):
        ret = datetime_utils.parse_date_delta(1)
        assert ret == WHEN + datetime.timedelta(0, 1)

def test_serialize_date_delta():
    """Testing datetime_utils.serialize_date_delta
    We need to verify the following scenarios:
        * if we pass something that's not an int or float, it should delegate
          the task to serialize_date
    """
    assert datetime_utils.serialize_date_delta(1) == '1'
    assert datetime_utils.serialize_date_delta(1.5) == '1'
    ret = datetime_utils.serialize_date_delta('Mon, 20 Nov 1995 19:12:08 GMT')
    assert type(ret) is (str)
    assert ret == 'Mon, 20 Nov 1995 19:12:08 GMT'

def test_timedelta_to_seconds():
    val = datetime.timedelta(86400)
    result = datetime_utils.timedelta_to_seconds(val)
    assert result == 7464960000


class _NowRestorer(object):
    def __init__(self, new_now):
        self._new_now = new_now
        self._old_now = None

    def __enter__(self):
        import webob.datetime_utils
        self._old_now = webob.datetime_utils._now
        webob.datetime_utils._now = lambda: self._new_now

    def __exit__(self, exc_type, exc_value, traceback):
        import webob.datetime_utils
        webob.datetime_utils._now = self._old_now



import unittest
import io
import socket

class TestSendRequest(unittest.TestCase):
    def _getTargetClass(self):
        from webob.client import SendRequest
        return SendRequest

    def _makeOne(self, **kw):
        cls = self._getTargetClass()
        return cls(**kw)

    def _makeEnviron(self, extra=None):
        environ = {
            'wsgi.url_scheme':'http',
            'SERVER_NAME':'localhost',
            'HTTP_HOST':'localhost:80',
            'SERVER_PORT':'80',
            'wsgi.input':io.BytesIO(),
            'CONTENT_LENGTH':0,
            'REQUEST_METHOD':'GET',
            }
        if extra is not None:
            environ.update(extra)
        return environ

    def test___call___unknown_scheme(self):
        environ = self._makeEnviron({'wsgi.url_scheme':'abc'})
        inst = self._makeOne()
        self.assertRaises(ValueError, inst, environ, None)

    def test___call___gardenpath(self):
        environ = self._makeEnviron()
        response = DummyResponse('msg')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '200 OK')
            self.assertEqual(headers, [])
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertEqual(list(iterable), [b'foo'])

    def test___call___no_servername_no_http_host(self):
        environ = self._makeEnviron()
        del environ['SERVER_NAME']
        del environ['HTTP_HOST']
        response = DummyResponse('msg')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        self.assertRaises(ValueError, inst, environ, None)

    def test___call___no_servername_colon_not_in_host_http(self):
        environ = self._makeEnviron()
        del environ['SERVER_NAME']
        environ['HTTP_HOST'] = 'localhost'
        response = DummyResponse('msg')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '200 OK')
            self.assertEqual(headers, [])
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertEqual(list(iterable), [b'foo'])
        self.assertEqual(environ['SERVER_NAME'], 'localhost')
        self.assertEqual(environ['SERVER_PORT'], '80')

    def test___call___no_servername_colon_not_in_host_https(self):
        environ = self._makeEnviron()
        del environ['SERVER_NAME']
        environ['HTTP_HOST'] = 'localhost'
        environ['wsgi.url_scheme'] = 'https'
        response = DummyResponse('msg')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPSConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '200 OK')
            self.assertEqual(headers, [])
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertEqual(list(iterable), [b'foo'])
        self.assertEqual(environ['SERVER_NAME'], 'localhost')
        self.assertEqual(environ['SERVER_PORT'], '443')

    def test___call___no_content_length(self):
        environ = self._makeEnviron()
        del environ['CONTENT_LENGTH']
        response = DummyResponse('msg')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '200 OK')
            self.assertEqual(headers, [])
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertEqual(list(iterable), [b'foo'])

    def test___call___with_webob_client_timeout_and_timeout_supported(self):
        environ = self._makeEnviron()
        environ['webob.client.timeout'] = 10
        response = DummyResponse('msg')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '200 OK')
            self.assertEqual(headers, [])
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertEqual(list(iterable), [b'foo'])
        self.assertEqual(conn_factory.kw, {'timeout':10})

    def test___call___bad_content_length(self):
        environ = self._makeEnviron({'CONTENT_LENGTH':'abc'})
        response = DummyResponse('msg')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '200 OK')
            self.assertEqual(headers, [])
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertEqual(list(iterable), [b'foo'])

    def test___call___with_socket_timeout(self):
        environ = self._makeEnviron()
        response = socket.timeout()
        response.msg = 'msg'
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '504 Gateway Timeout')
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertTrue(list(iterable)[0].startswith(b'504'))

    def test___call___with_socket_error_neg2(self):
        environ = self._makeEnviron()
        response = socket.error(-2)
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '502 Bad Gateway')
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertTrue(list(iterable)[0].startswith(b'502'))

    def test___call___with_socket_error_ENODATA(self):
        import errno
        environ = self._makeEnviron()
        if not hasattr(errno, 'ENODATA'):
            # no ENODATA on win
            return
        response = socket.error(errno.ENODATA)
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '502 Bad Gateway')
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertTrue(list(iterable)[0].startswith(b'502'))

    def test___call___with_socket_error_unknown(self):
        environ = self._makeEnviron()
        response = socket.error('nope')
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '502 Bad Gateway')
            inst.start_response_called = True
        self.assertRaises(socket.error, inst, environ, start_response)

    def test___call___nolength(self):
        environ = self._makeEnviron()
        response = DummyResponse('msg', None)
        conn_factory = DummyConnectionFactory(response)
        inst = self._makeOne(HTTPConnection=conn_factory)
        def start_response(status, headers):
            self.assertEqual(status, '200 OK')
            self.assertEqual(headers, [])
            inst.start_response_called = True
        iterable = inst(environ, start_response)
        self.assertTrue(inst.start_response_called)
        self.assertEqual(list(iterable), [b'foo'])
        self.assertEqual(response.length, None)

class DummyMessage(object):
    def __init__(self, msg):
        self.msg = msg
        self.headers = self._headers = {}

class DummyResponse(object):
    def __init__(self, msg, headerval='10'):
        self.msg = DummyMessage(msg)
        self.status = '200'
        self.reason = 'OK'
        self.headerval = headerval

    def getheader(self, name):
        return self.headerval

    def read(self, length=None):
        self.length = length
        return b'foo'

class DummyConnectionFactory(object):
    def __init__(self, result=None):
        self.result = result
        self.closed = False

    def __call__(self, hostport, **kw):
        self.hostport = hostport
        self.kw = kw
        self.request = DummyRequestFactory(hostport, **kw)
        return self

    def getresponse(self):
        if isinstance(self.result, Exception):
            raise self.result
        return self.result

    def close(self):
        self.closed = True

class DummyRequestFactory(object):
    def __init__(self, hostport, **kw):
        self.hostport = hostport
        self.kw = kw

    def __call__(self, method, path, body, headers):
        return self



from datetime import tzinfo
from datetime import timedelta

import pytest

from webob.compat import (
    native_,
    text_,
    )

from webob.request import Request

class GMT(tzinfo):
    """UTC"""
    ZERO = timedelta(0)
    def utcoffset(self, dt):
        return self.ZERO

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return self.ZERO


class MockDescriptor:
    _val = 'avalue'
    def __get__(self, obj, type=None):
        return self._val
    def __set__(self, obj, val):
        self._val = val
    def __delete__(self, obj):
        self._val = None


def test_environ_getter_docstring():
    from webob.descriptors import environ_getter
    desc = environ_getter('akey')
    assert desc.__doc__ == "Gets and sets the ``akey`` key in the environment."

def test_environ_getter_nodefault_keyerror():
    from webob.descriptors import environ_getter
    req = Request.blank('/')
    desc = environ_getter('akey')
    with pytest.raises(KeyError):
        desc.fget(req)

def test_environ_getter_nodefault_fget():
    from webob.descriptors import environ_getter
    req = Request.blank('/')
    desc = environ_getter('akey')
    desc.fset(req, 'bar')
    assert req.environ['akey'] == 'bar'

def test_environ_getter_nodefault_fdel():
    from webob.descriptors import environ_getter
    desc = environ_getter('akey')
    assert desc.fdel is None

def test_environ_getter_default_fget():
    from webob.descriptors import environ_getter
    req = Request.blank('/')
    desc = environ_getter('akey', default='the_default')
    assert desc.fget(req) == 'the_default'

def test_environ_getter_default_fset():
    from webob.descriptors import environ_getter
    req = Request.blank('/')
    desc = environ_getter('akey', default='the_default')
    desc.fset(req, 'bar')
    assert req.environ['akey'] == 'bar'

def test_environ_getter_default_fset_none():
    from webob.descriptors import environ_getter
    req = Request.blank('/')
    desc = environ_getter('akey', default='the_default')
    desc.fset(req, 'baz')
    desc.fset(req, None)
    assert 'akey' not in req.environ

def test_environ_getter_default_fdel():
    from webob.descriptors import environ_getter
    req = Request.blank('/')
    desc = environ_getter('akey', default='the_default')
    desc.fset(req, 'baz')
    assert 'akey' in req.environ
    desc.fdel(req)
    assert 'akey' not in req.environ

def test_environ_getter_rfc_section():
    from webob.descriptors import environ_getter
    desc = environ_getter('HTTP_X_AKEY', rfc_section='14.3')
    assert desc.__doc__ == "Gets and sets the ``X-Akey`` header "\
        "(`HTTP spec section 14.3 "\
        "<http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.3>`_)."


def test_upath_property_fget():
    from webob.descriptors import upath_property
    req = Request.blank('/')
    desc = upath_property('akey')
    assert desc.fget(req) == ''

def test_upath_property_fset():
    from webob.descriptors import upath_property
    req = Request.blank('/')
    desc = upath_property('akey')
    desc.fset(req, 'avalue')
    assert desc.fget(req) == 'avalue'

def test_header_getter_doc():
    from webob.descriptors import header_getter
    desc = header_getter('X-Header', '14.3')
    assert('http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.3'
           in desc.__doc__)
    assert '``X-Header`` header' in desc.__doc__

def test_header_getter_fget():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    assert desc.fget(resp) is None

def test_header_getter_fset():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, 'avalue')
    assert desc.fget(resp) == 'avalue'

def test_header_getter_fset_none():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, 'avalue')
    desc.fset(resp, None)
    assert desc.fget(resp) is None

def test_header_getter_fset_text():
    from webob.compat import text_
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, text_('avalue'))
    assert desc.fget(resp) == 'avalue'

def test_header_getter_fset_text_control_chars():
    from webob.compat import text_
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    with pytest.raises(ValueError):
        desc.fset(resp, text_('\n'))

def test_header_getter_fdel():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, 'avalue2')
    desc.fdel(resp)
    assert desc.fget(resp) is None

def test_header_getter_unicode_fget_none():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    assert desc.fget(resp) is None

def test_header_getter_unicode_fget():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, 'avalue')
    assert desc.fget(resp) == 'avalue'

def test_header_getter_unicode_fset_none():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, None)
    assert desc.fget(resp) is None

def test_header_getter_unicode_fset():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, 'avalue2')
    assert desc.fget(resp) == 'avalue2'

def test_header_getter_unicode_fdel():
    from webob.descriptors import header_getter
    from webob import Response
    resp = Response('aresp')
    desc = header_getter('AHEADER', '14.3')
    desc.fset(resp, 'avalue3')
    desc.fdel(resp)
    assert desc.fget(resp) is None

def test_converter_not_prop():
    from webob.descriptors import converter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    with pytest.raises(AssertionError):
        converter(
            ('CONTENT_LENGTH', None, '14.13'),
            parse_int_safe, serialize_int,
            'int')

def test_converter_with_name_docstring():
    from webob.descriptors import converter
    from webob.descriptors import environ_getter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    desc = converter(
        environ_getter('CONTENT_LENGTH', '666', '14.13'),
        parse_int_safe, serialize_int, 'int')

    assert 'http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.13' in desc.__doc__
    assert '``Content-Length`` header' in desc.__doc__

def test_converter_with_name_fget():
    from webob.descriptors import converter
    from webob.descriptors import environ_getter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    req = Request.blank('/')
    desc = converter(
        environ_getter('CONTENT_LENGTH', '666', '14.13'),
        parse_int_safe, serialize_int, 'int')
    assert desc.fget(req) == 666

def test_converter_with_name_fset():
    from webob.descriptors import converter
    from webob.descriptors import environ_getter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    req = Request.blank('/')
    desc = converter(
        environ_getter('CONTENT_LENGTH', '666', '14.13'),
        parse_int_safe, serialize_int, 'int')
    desc.fset(req, '999')
    assert desc.fget(req) == 999

def test_converter_without_name_fget():
    from webob.descriptors import converter
    from webob.descriptors import environ_getter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    req = Request.blank('/')
    desc = converter(
        environ_getter('CONTENT_LENGTH', '666', '14.13'),
        parse_int_safe, serialize_int)
    assert desc.fget(req) == 666

def test_converter_without_name_fset():
    from webob.descriptors import converter
    from webob.descriptors import environ_getter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    req = Request.blank('/')
    desc = converter(
        environ_getter('CONTENT_LENGTH', '666', '14.13'),
        parse_int_safe, serialize_int)
    desc.fset(req, '999')
    assert desc.fget(req) == 999

def test_converter_none_for_wrong_type():
    from webob.descriptors import converter
    from webob.descriptors import environ_getter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    req = Request.blank('/')
    desc = converter(
        environ_getter('CONTENT_LENGTH', 'sixsixsix', '14.13'),
        parse_int_safe, serialize_int, 'int')
    assert desc.fget(req) is None

def test_converter_delete():
    from webob.descriptors import converter
    from webob.descriptors import environ_getter
    from webob.descriptors import parse_int_safe
    from webob.descriptors import serialize_int
    req = Request.blank('/')
    desc = converter(
        environ_getter('CONTENT_LENGTH', '666', '14.13'),
        parse_int_safe, serialize_int, 'int')
    with pytest.raises(KeyError):
        desc.fdel(req)

def test_list_header():
    from webob.descriptors import list_header
    desc = list_header('CONTENT_LENGTH', '14.13')
    assert type(desc) == property

def test_parse_list_single():
    from webob.descriptors import parse_list
    result = parse_list('avalue')
    assert result == ('avalue',)

def test_parse_list_multiple():
    from webob.descriptors import parse_list
    result = parse_list('avalue,avalue2')
    assert result == ('avalue', 'avalue2')

def test_parse_list_none():
    from webob.descriptors import parse_list
    result = parse_list(None)
    assert result is None

def test_parse_list_unicode_single():
    from webob.descriptors import parse_list
    result = parse_list('avalue')
    assert result == ('avalue',)

def test_parse_list_unicode_multiple():
    from webob.descriptors import parse_list
    result = parse_list('avalue,avalue2')
    assert result == ('avalue', 'avalue2')

def test_serialize_list():
    from webob.descriptors import serialize_list
    result = serialize_list(('avalue', 'avalue2'))
    assert result == 'avalue, avalue2'

def test_serialize_list_string():
    from webob.descriptors import serialize_list
    result = serialize_list('avalue')
    assert result == 'avalue'

def test_serialize_list_unicode():
    from webob.descriptors import serialize_list
    result = serialize_list('avalue')
    assert result == 'avalue'

def test_converter_date():
    import datetime
    from webob.descriptors import converter_date
    from webob.descriptors import environ_getter
    req = Request.blank('/')
    UTC = GMT()
    desc = converter_date(environ_getter(
        "HTTP_DATE", "Tue, 15 Nov 1994 08:12:31 GMT", "14.8"))
    assert desc.fget(req) == datetime.datetime(1994, 11, 15, 8, 12, 31, tzinfo=UTC)

def test_converter_date_docstring():
    from webob.descriptors import converter_date
    from webob.descriptors import environ_getter
    desc = converter_date(environ_getter(
        "HTTP_DATE", "Tue, 15 Nov 1994 08:12:31 GMT", "14.8"))
    assert 'http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.8' in desc.__doc__
    assert '``Date`` header' in desc.__doc__


def test_date_header_fget_none():
    from webob import Response
    from webob.descriptors import date_header
    resp = Response('aresponse')
    desc = date_header('HTTP_DATE', "14.8")
    assert desc.fget(resp) is None

def test_date_header_fset_fget():
    import datetime
    from webob import Response
    from webob.descriptors import date_header
    resp = Response('aresponse')
    UTC = GMT()
    desc = date_header('HTTP_DATE', "14.8")
    desc.fset(resp, "Tue, 15 Nov 1994 08:12:31 GMT")
    assert desc.fget(resp) == datetime.datetime(1994, 11, 15, 8, 12, 31, tzinfo=UTC)

def test_date_header_fdel():
    from webob import Response
    from webob.descriptors import date_header
    resp = Response('aresponse')
    desc = date_header('HTTP_DATE', "14.8")
    desc.fset(resp, "Tue, 15 Nov 1994 08:12:31 GMT")
    desc.fdel(resp)
    assert desc.fget(resp) is None

def test_deprecated_property():
    from webob.descriptors import deprecated_property

    class Foo(object):
        pass
    Foo.attr = deprecated_property('attr', 'attr', 'whatever', '1.2')
    foo = Foo()
    with pytest.raises(DeprecationWarning):
        getattr(foo, 'attr')
    with pytest.raises(DeprecationWarning):
        setattr(foo, 'attr', {})
    with pytest.raises(DeprecationWarning):
        delattr(foo, 'attr')

def test_parse_etag_response():
    from webob.descriptors import parse_etag_response
    etresp = parse_etag_response("etag")
    assert etresp == "etag"

def test_parse_etag_response_quoted():
    from webob.descriptors import parse_etag_response
    etresp = parse_etag_response('"etag"')
    assert etresp == "etag"

def test_parse_etag_response_is_none():
    from webob.descriptors import parse_etag_response
    etresp = parse_etag_response(None)
    assert etresp is None

def test_serialize_etag_response():
    from webob.descriptors import serialize_etag_response
    etresp = serialize_etag_response("etag")
    assert etresp == '"etag"'

def test_serialize_if_range_string():
    from webob.descriptors import serialize_if_range
    val = serialize_if_range("avalue")
    assert val == "avalue"

def test_serialize_if_range_unicode():
    from webob.descriptors import serialize_if_range
    val = serialize_if_range("avalue")
    assert val == "avalue"

def test_serialize_if_range_datetime():
    import datetime
    from webob.descriptors import serialize_if_range
    UTC = GMT()
    val = serialize_if_range(datetime.datetime(1994, 11, 15, 8, 12, 31, tzinfo=UTC))
    assert val, "Tue == 15 Nov 1994 08:12:31 GMT"

def test_serialize_if_range_other():
    from webob.descriptors import serialize_if_range
    val = serialize_if_range(123456)
    assert val == '123456'

def test_parse_range_none():
    from webob.descriptors import parse_range
    assert parse_range(None) is None

def test_parse_range_type():
    from webob.byterange import Range
    from webob.descriptors import parse_range
    val = parse_range("bytes=1-500")
    assert type(val) is type(Range.parse("bytes=1-500"))

def test_parse_range_values():
    from webob.byterange import Range
    range = Range.parse("bytes=1-500")
    assert range.start == 1
    assert range.end == 501

def test_serialize_range_none():
    from webob.descriptors import serialize_range
    val = serialize_range(None)
    assert val is None

def test_serialize_range():
    from webob.descriptors import serialize_range
    val = serialize_range((1, 500))
    assert val == 'bytes=1-499'

def test_parse_int_none():
    from webob.descriptors import parse_int
    val = parse_int(None)
    assert val is None

def test_parse_int_emptystr():
    from webob.descriptors import parse_int
    val = parse_int('')
    assert val is None

def test_parse_int():
    from webob.descriptors import parse_int
    val = parse_int('123')
    assert val == 123

def test_parse_int_invalid():
    from webob.descriptors import parse_int
    with pytest.raises(ValueError):
        parse_int('abc')

def test_parse_int_safe_none():
    from webob.descriptors import parse_int_safe
    assert parse_int_safe(None) is None

def test_parse_int_safe_emptystr():
    from webob.descriptors import parse_int_safe
    assert parse_int_safe('') is None

def test_parse_int_safe():
    from webob.descriptors import parse_int_safe
    assert parse_int_safe('123') == 123

def test_parse_int_safe_invalid():
    from webob.descriptors import parse_int_safe
    assert parse_int_safe('abc') is None

def test_serialize_int():
    from webob.descriptors import serialize_int
    assert serialize_int is str

def test_parse_content_range_none():
    from webob.descriptors import parse_content_range
    assert parse_content_range(None) is None

def test_parse_content_range_emptystr():
    from webob.descriptors import parse_content_range
    assert parse_content_range(' ') is None

def test_parse_content_range_length():
    from webob.byterange import ContentRange
    from webob.descriptors import parse_content_range
    val = parse_content_range("bytes 0-499/1234")
    assert val.length == ContentRange.parse("bytes 0-499/1234").length

def test_parse_content_range_start():
    from webob.byterange import ContentRange
    from webob.descriptors import parse_content_range
    val = parse_content_range("bytes 0-499/1234")
    assert val.start == ContentRange.parse("bytes 0-499/1234").start

def test_parse_content_range_stop():
    from webob.byterange import ContentRange
    from webob.descriptors import parse_content_range
    val = parse_content_range("bytes 0-499/1234")
    assert val.stop == ContentRange.parse("bytes 0-499/1234").stop

def test_serialize_content_range_none():
    from webob.descriptors import serialize_content_range
    assert serialize_content_range(None) == 'None' ### XXX: Seems wrong

def test_serialize_content_range_emptystr():
    from webob.descriptors import serialize_content_range
    assert serialize_content_range('') is None

def test_serialize_content_range_invalid():
    from webob.descriptors import serialize_content_range
    with pytest.raises(ValueError):
        serialize_content_range((1,))

def test_serialize_content_range_asterisk():
    from webob.descriptors import serialize_content_range
    assert serialize_content_range((0, 500)) == 'bytes 0-499/*'

def test_serialize_content_range_defined():
    from webob.descriptors import serialize_content_range
    assert serialize_content_range((0, 500, 1234)) == 'bytes 0-499/1234'

def test_parse_auth_params_leading_capital_letter():
    from webob.descriptors import parse_auth_params
    val = parse_auth_params('Basic Realm=WebOb')
    assert val == {'ealm': 'WebOb'}

def test_parse_auth_params_trailing_capital_letter():
    from webob.descriptors import parse_auth_params
    val = parse_auth_params('Basic realM=WebOb')
    assert val == {}

def test_parse_auth_params_doublequotes():
    from webob.descriptors import parse_auth_params
    val = parse_auth_params('Basic realm="Web Object"')
    assert val == {'realm': 'Web Object'}

def test_parse_auth_params_multiple_values():
    from webob.descriptors import parse_auth_params
    val = parse_auth_params("foo='blah &&234', qop=foo, nonce='qwerty1234'")
    assert val == {'nonce': "'qwerty1234'", 'foo': "'blah &&234'", 'qop': 'foo'}

def test_parse_auth_params_truncate_on_comma():
    from webob.descriptors import parse_auth_params
    val = parse_auth_params("Basic realm=WebOb,this_will_truncate")
    assert val == {'realm': 'WebOb'}

def test_parse_auth_params_emptystr():
    from webob.descriptors import parse_auth_params
    assert parse_auth_params('') == {}

def test_parse_auth_params_bad_whitespace():
    from webob.descriptors import parse_auth_params
    assert parse_auth_params('a= "2 ", b =3, c=4 ') == {
        'a': '2 ',
        'b': '3',
        'c': '4'
    }

def test_authorization2():
    from webob.descriptors import parse_auth_params
    for s, d in [
            ('x=y', {'x': 'y'}),
            ('x="y"', {'x': 'y'}),
            ('x=y,z=z', {'x': 'y', 'z': 'z'}),
            ('x=y, z=z', {'x': 'y', 'z': 'z'}),
            ('x="y",z=z', {'x': 'y', 'z': 'z'}),
            ('x="y", z=z', {'x': 'y', 'z': 'z'}),
            ('x="y,x", z=z', {'x': 'y,x', 'z': 'z'}),
            ]:
        assert parse_auth_params(s) == d

def test_parse_auth_none():
    from webob.descriptors import parse_auth
    assert parse_auth(None) is None

def test_parse_auth_emptystr():
    from webob.descriptors import parse_auth
    assert parse_auth('') == ('', '')

def test_parse_auth_bearer():
    from webob.descriptors import parse_auth
    assert parse_auth('Bearer token').authtype == 'Bearer'
    assert parse_auth('Bearer token').params == 'token'

def test_parse_auth_unknown_nospace():
    from webob.descriptors import parse_auth
    assert parse_auth('NoSpace') == ('NoSpace', '')

def test_parse_auth_known_nospace():
    from webob.descriptors import parse_auth
    assert parse_auth('Digest') == ('Digest', {})

def test_parse_auth_basic():
    from webob.descriptors import parse_auth
    assert parse_auth("Basic realm=WebOb") == ('Basic', 'realm=WebOb')

def test_parse_auth_basic_quoted():
    from webob.descriptors import parse_auth
    assert parse_auth('Basic realm="Web Ob"') == ('Basic', {'realm': 'Web Ob'})

def test_parse_auth_basic_quoted_multiple_unknown():
    from webob.descriptors import parse_auth
    assert parse_auth("foo='blah &&234', qop=foo, nonce='qwerty1234'") == (
        "foo='blah",
        "&&234', qop=foo, nonce='qwerty1234'"
    )

def test_parse_auth_basic_quoted_known_multiple():
    from webob.descriptors import parse_auth
    assert parse_auth("Basic realm='blah &&234', qop=foo, nonce='qwerty1234'") == (
        'Basic',
        "realm='blah &&234', qop=foo, nonce='qwerty1234'"
    )

def test_serialize_auth_none():
    from webob.descriptors import serialize_auth
    assert serialize_auth(None) is None

def test_serialize_auth_emptystr():
    from webob.descriptors import serialize_auth
    assert serialize_auth('') == ''

def test_serialize_auth_str():
    from webob.descriptors import serialize_auth
    assert serialize_auth('some string') == 'some string'

def test_serialize_auth_parsed_emptystr():
    from webob.descriptors import serialize_auth
    assert serialize_auth(('', '')) == ' '

def test_serialize_auth_parsed_unknown_nospace():
    from webob.descriptors import serialize_auth
    assert serialize_auth(('NoSpace', '')) == 'NoSpace '

def test_serialize_auth_parsed_known_nospace():
    from webob.descriptors import serialize_auth
    assert serialize_auth(('Digest', {})) == 'Digest '

def test_serialize_auth_basic_quoted():
    from webob.descriptors import serialize_auth
    val = serialize_auth(('Basic', 'realm="WebOb"'))
    assert val == 'Basic realm="WebOb"'

def test_serialize_auth_digest_multiple():
    from webob.descriptors import serialize_auth
    val = serialize_auth(('Digest', 'realm="WebOb", nonce=abcde12345, qop=foo'))
    flags = val[len('Digest'):]
    result = sorted([ x.strip() for x in flags.split(',') ])
    assert result == ['nonce=abcde12345', 'qop=foo', 'realm="WebOb"']

def test_serialize_auth_digest_tuple():
    from webob.descriptors import serialize_auth
    val = serialize_auth(('Digest', {'realm':'"WebOb"', 'nonce':'abcde12345', 'qop':'foo'}))
    flags = val[len('Digest'):]
    result = sorted([ x.strip() for x in flags.split(',') ])
    assert result == ['nonce="abcde12345"', 'qop="foo"', 'realm=""WebOb""']


_nodefault = object()

class _TestEnvironDecoder(object):
    def _callFUT(self, key, default=_nodefault, rfc_section=None,
                 encattr=None):
        from webob.descriptors import environ_decoder
        if default is _nodefault:
            return environ_decoder(key, rfc_section=rfc_section,
                                   encattr=encattr)
        else:
            return environ_decoder(key, default=default,
                                   rfc_section=rfc_section, encattr=encattr)

    def test_docstring(self):
        desc = self._callFUT('akey')
        assert desc.__doc__ == "Gets and sets the ``akey`` key in the environment."

    def test_nodefault_keyerror(self):
        req = self._makeRequest()
        desc = self._callFUT('akey')
        with pytest.raises(KeyError):
            desc.fget(req)

    def test_nodefault_fget(self):
        req = self._makeRequest()
        desc = self._callFUT('akey')
        desc.fset(req, 'bar')
        assert req.environ['akey'] == 'bar'

    def test_nodefault_fdel(self):
        desc = self._callFUT('akey')
        assert desc.fdel is None

    def test_default_fget(self):
        req = self._makeRequest()
        desc = self._callFUT('akey', default='the_default')
        assert desc.fget(req) == 'the_default'

    def test_default_fset(self):
        req = self._makeRequest()
        desc = self._callFUT('akey', default='the_default')
        desc.fset(req, 'bar')
        assert req.environ['akey'] == 'bar'

    def test_default_fset_none(self):
        req = self._makeRequest()
        desc = self._callFUT('akey', default='the_default')
        desc.fset(req, 'baz')
        desc.fset(req, None)
        assert 'akey' not in req.environ

    def test_default_fdel(self):
        req = self._makeRequest()
        desc = self._callFUT('akey', default='the_default')
        desc.fset(req, 'baz')
        assert 'akey' in req.environ
        desc.fdel(req)
        assert 'akey' not in req.environ

    def test_rfc_section(self):
        desc = self._callFUT('HTTP_X_AKEY', rfc_section='14.3')
        assert desc.__doc__ == "Gets and sets the ``X-Akey`` header "\
            "(`HTTP spec section 14.3 "\
            "<http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.3>`_)."

    def test_fset_nonascii(self):
        desc = self._callFUT('HTTP_X_AKEY', encattr='url_encoding')
        req = self._makeRequest()
        desc.fset(req, text_(b'\xc3\xab', 'utf-8'))
        assert req.environ['HTTP_X_AKEY'] == native_(b'\xc3\xab', 'latin-1')

class TestEnvironDecoder(_TestEnvironDecoder):
    def _makeRequest(self):
        from webob.request import BaseRequest
        req = BaseRequest.blank('/')
        return req

    def test_fget_nonascii(self):
        desc = self._callFUT('HTTP_X_AKEY', encattr='url_encoding')
        req = self._makeRequest()
        req.environ['HTTP_X_AKEY'] = native_(b'\xc3\xab')
        result = desc.fget(req)
        assert result == text_(b'\xc3\xab', 'utf-8')

class TestEnvironDecoderLegacy(_TestEnvironDecoder):
    def _makeRequest(self):
        from webob.request import LegacyRequest
        req = LegacyRequest.blank('/')
        return req

    def test_fget_nonascii(self):
        desc = self._callFUT('HTTP_X_AKEY', encattr='url_encoding')
        req = self._makeRequest()
        req.environ['HTTP_X_AKEY'] = native_(b'\xc3\xab', 'latin-1')
        result = desc.fget(req)
        assert result == native_(b'\xc3\xab', 'latin-1')

    def test_default_fget_nonascii(self):
        req = self._makeRequest()
        desc = self._callFUT('akey', default=b'the_default')
        assert desc.fget(req).__class__ == bytes



import unittest
from webob.response import Response

class Test_warn_deprecation(unittest.TestCase):
    def setUp(self):
        import warnings
        self.oldwarn = warnings.warn
        warnings.warn = self._warn
        self.warnings = []

    def tearDown(self):
        import warnings
        warnings.warn = self.oldwarn
        del self.warnings

    def _callFUT(self, text, version, stacklevel):
        from webob.util import warn_deprecation
        return warn_deprecation(text, version, stacklevel)

    def _warn(self, text, type, stacklevel=1):
        self.warnings.append(locals())

    def test_multidict_update_warning(self):
        # test warning when duplicate keys are passed
        r = Response()
        r.headers.update([
            ('Set-Cookie', 'a=b'),
            ('Set-Cookie', 'x=y'),
        ])
        self.assertEqual(len(self.warnings), 1)
        deprecation_warning = self.warnings[0]
        self.assertEqual(deprecation_warning['type'], UserWarning)
        assert 'Consider using .extend()' in deprecation_warning['text']

    def test_multidict_update_warning_unnecessary(self):
        # no warning on normal operation
        r = Response()
        r.headers.update([('Set-Cookie', 'a=b')])
        self.assertEqual(len(self.warnings), 0)

    def test_warn_deprecation(self):
        v = '1.3.0'
        from webob.util import warn_deprecation
        self.assertRaises(DeprecationWarning, warn_deprecation, 'foo', v[:3], 1)

    def test_warn_deprecation_future_version(self):
        v = '9.9.9'
        from webob.util import warn_deprecation
        warn_deprecation('foo', v[:3], 1)
        self.assertEqual(len(self.warnings), 1)

class Test_strings_differ(unittest.TestCase):
    def _callFUT(self, *args, **kw):
        from webob.util import strings_differ
        return strings_differ(*args, **kw)

    def test_it(self):
        self.assertFalse(self._callFUT(b'foo', b'foo'))
        self.assertTrue(self._callFUT(b'123', b'345'))
        self.assertTrue(self._callFUT(b'1234', b'123'))
        self.assertTrue(self._callFUT(b'123', b'1234'))

    def test_it_with_internal_comparator(self):
        result = self._callFUT(b'foo', b'foo', compare_digest=None)
        self.assertFalse(result)

        result = self._callFUT(b'123', b'abc', compare_digest=None)
        self.assertTrue(result)

    def test_it_with_external_comparator(self):
        class DummyComparator(object):
            called = False
            def __init__(self, ret_val):
                self.ret_val = ret_val

            def __call__(self, a, b):
                self.called = True
                return self.ret_val

        dummy_compare = DummyComparator(True)
        result = self._callFUT(b'foo', b'foo', compare_digest=dummy_compare)
        self.assertTrue(dummy_compare.called)
        self.assertFalse(result)

        dummy_compare = DummyComparator(False)
        result = self._callFUT(b'123', b'345', compare_digest=dummy_compare)
        self.assertTrue(dummy_compare.called)
        self.assertTrue(result)

        dummy_compare = DummyComparator(False)
        result = self._callFUT(b'abc', b'abc', compare_digest=dummy_compare)
        self.assertTrue(dummy_compare.called)
        self.assertTrue(result)




import pytest

from webob import headers

def test_ResponseHeaders_delitem_notpresent():
    """Deleting a missing key from ResponseHeaders should raise a KeyError"""
    d = headers.ResponseHeaders()
    with pytest.raises(KeyError):
        d.__delitem__('b')

def test_ResponseHeaders_delitem_present():
    """
    Deleting a present key should not raise an error at all
    """
    d = headers.ResponseHeaders(a=1)
    del d['a']
    assert 'a' not in d

def test_ResponseHeaders_setdefault():
    """Testing set_default for ResponseHeaders"""
    d = headers.ResponseHeaders(a=1)
    res = d.setdefault('b', 1)
    assert res == d['b'] == 1
    res = d.setdefault('b', 10)
    assert res == d['b'] == 1
    res = d.setdefault('B', 10)
    assert res == d['b'] == d['B'] == 1

def test_ResponseHeader_pop():
    """Testing if pop return TypeError when more than len(*args)>1 plus other
    assorted tests"""
    d = headers.ResponseHeaders(a=1, b=2, c=3, d=4)
    with pytest.raises(TypeError):
        d.pop('a', 'z', 'y')
    assert d.pop('a') == 1
    assert 'a' not in d
    assert d.pop('B') == 2
    assert 'b' not in d
    assert d.pop('c', 'u') == 3
    assert 'c' not in d
    assert d.pop('e', 'u') == 'u'
    assert 'e' not in d
    with pytest.raises(KeyError):
        d.pop('z')

def test_ResponseHeaders_getitem_miss():
    d = headers.ResponseHeaders()
    with pytest.raises(KeyError):
        d.__getitem__('a')

def test_ResponseHeaders_getall():
    d = headers.ResponseHeaders()
    d.add('a', 1)
    d.add('a', 2)
    result = d.getall('a')
    assert result == [1,2]

def test_ResponseHeaders_mixed():
    d = headers.ResponseHeaders()
    d.add('a', 1)
    d.add('a', 2)
    d['b'] = 1
    result = d.mixed()
    assert result == {'a':[1,2], 'b':1}

def test_ResponseHeaders_setitem_scalar_replaces_seq():
    d = headers.ResponseHeaders()
    d.add('a', 2)
    d['a'] = 1
    result = d.getall('a')
    assert result == [1]

def test_ResponseHeaders_contains():
    d = headers.ResponseHeaders()
    d['a'] = 1
    assert 'a' in d
    assert not 'b' in d

def test_EnvironHeaders_delitem():
    d = headers.EnvironHeaders({'CONTENT_LENGTH': '10'})
    del d['CONTENT-LENGTH']
    assert not d
    with pytest.raises(KeyError):
        d.__delitem__('CONTENT-LENGTH')

def test_EnvironHeaders_getitem():
    d = headers.EnvironHeaders({'CONTENT_LENGTH': '10'})
    assert d['CONTENT-LENGTH'] == '10'

def test_EnvironHeaders_setitem():
    d = headers.EnvironHeaders({})
    d['abc'] = '10'
    assert d['abc'] == '10'

def test_EnvironHeaders_contains():
    d = headers.EnvironHeaders({})
    d['a'] = '10'
    assert 'a' in d
    assert 'b' not in d

def test__trans_key_not_basestring():
    result = headers._trans_key(None)
    assert result == None

def test__trans_key_not_a_header():
    result = headers._trans_key('')
    assert result == None

def test__trans_key_key2header():
    result = headers._trans_key('CONTENT_TYPE')
    assert result == 'Content-Type'

def test__trans_key_httpheader():
    result = headers._trans_key('HTTP_FOO_BAR')
    assert result == 'Foo-Bar'



import unittest
import doctest

def test_suite():
    flags = doctest.ELLIPSIS|doctest.NORMALIZE_WHITESPACE
    return unittest.TestSuite((
        doctest.DocFileSuite('test_request.txt', optionflags=flags),
        doctest.DocFileSuite('test_response.txt', optionflags=flags),
        doctest.DocFileSuite('test_dec.txt', optionflags=flags),
        doctest.DocFileSuite('do-it-yourself.txt', optionflags=flags),
        doctest.DocFileSuite('file-example.txt', optionflags=flags),
        doctest.DocFileSuite('index.txt', optionflags=flags),
        doctest.DocFileSuite('reference.txt', optionflags=flags),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



import pytest

import json

from webob.request import Request
from webob.dec import wsgify
from webob import exc as webob_exc

@wsgify
def method_not_allowed_app(req):
    if req.method != 'GET':
        raise webob_exc.HTTPMethodNotAllowed()
    return 'hello!'

def test_noescape_null():
    assert webob_exc.no_escape(None) == ''

def test_noescape_not_basestring():
    assert webob_exc.no_escape(42) == '42'

def test_noescape_unicode():
    class DummyUnicodeObject(object):
        def __unicode__(self):
            return '42'
    duo = DummyUnicodeObject()
    assert webob_exc.no_escape(duo) == '42'

def test_strip_tags_empty():
    assert webob_exc.strip_tags('') == ''

def test_strip_tags_newline_to_space():
    assert webob_exc.strip_tags('a\nb') == 'a b'

def test_strip_tags_zaps_carriage_return():
    assert webob_exc.strip_tags('a\rb') == 'ab'

def test_strip_tags_br_to_newline():
    assert webob_exc.strip_tags('a<br/>b') == 'a\nb'

def test_strip_tags_zaps_comments():
    assert webob_exc.strip_tags('a<!--b-->') == 'ab'

def test_strip_tags_zaps_tags():
    assert webob_exc.strip_tags('foo<bar>baz</bar>') == 'foobaz'

def test_HTTPException():
    _called = []
    _result = object()
    def _response(environ, start_response):
        _called.append((environ, start_response))
        return _result
    environ = {}
    start_response = object()
    exc = webob_exc.HTTPException('testing', _response)
    assert exc.wsgi_response is _response
    result = exc(environ, start_response)
    assert result is result
    assert _called == [(environ, start_response)]

def test_exception_with_unicode_data():
    req = Request.blank('/', method='POST')
    res = req.get_response(method_not_allowed_app)
    assert res.status_code == 405

def test_WSGIHTTPException_headers():
    exc = webob_exc.WSGIHTTPException(headers=[('Set-Cookie', 'a=1'),
                                     ('Set-Cookie', 'a=2')])
    mixed = exc.headers.mixed()
    assert mixed['set-cookie'] == ['a=1', 'a=2']

def test_WSGIHTTPException_w_body_template():
    from string import Template
    TEMPLATE = '$foo: $bar'
    exc = webob_exc.WSGIHTTPException(body_template=TEMPLATE)
    assert exc.body_template == TEMPLATE
    assert isinstance(exc.body_template_obj, Template)
    assert exc.body_template_obj.substitute({'foo': 'FOO', 'bar': 'BAR'}) == 'FOO: BAR'

def test_WSGIHTTPException_w_empty_body():
    class EmptyOnly(webob_exc.WSGIHTTPException):
        empty_body = True
    exc = EmptyOnly(content_type='text/plain', content_length=234)
    assert 'content_type' not in exc.__dict__
    assert 'content_length' not in exc.__dict__

def test_WSGIHTTPException___str__():
    exc1 = webob_exc.WSGIHTTPException(detail='Detail')
    assert str(exc1) == 'Detail'
    class Explain(webob_exc.WSGIHTTPException):
        explanation = 'Explanation'
    assert str(Explain()) == 'Explanation'

def test_WSGIHTTPException_plain_body_no_comment():
    class Explain(webob_exc.WSGIHTTPException):
        code = '999'
        title = 'Testing'
        explanation = 'Explanation'
    exc = Explain(detail='Detail')
    assert exc.plain_body({}) == '999 Testing\n\nExplanation\n\n Detail  '

def test_WSGIHTTPException_html_body_w_comment():
    class Explain(webob_exc.WSGIHTTPException):
        code = '999'
        title = 'Testing'
        explanation = 'Explanation'
    exc = Explain(detail='Detail', comment='Comment')
    assert exc.html_body({}) == (
        '<html>\n'
        ' <head>\n'
        '  <title>999 Testing</title>\n'
        ' </head>\n'
        ' <body>\n'
        '  <h1>999 Testing</h1>\n'
        '  Explanation<br /><br />\n'
        'Detail\n'
        '<!-- Comment -->\n\n'
        ' </body>\n'
        '</html>'
        )

def test_WSGIHTTPException_json_body_no_comment():
    class ValidationError(webob_exc.WSGIHTTPException):
        code = '422'
        title = 'Validation Failed'
        explanation = 'Validation of an attribute failed.'

    exc = ValidationError(detail='Attribute "xyz" is invalid.')
    body = exc.json_body({})
    assert json.loads(body) == {
        "code": "422 Validation Failed",
        "title": "Validation Failed",
        "message": "Validation of an attribute failed.<br /><br />\nAttribute"
                   ' "xyz" is invalid.\n\n',
    }

def test_WSGIHTTPException_respects_application_json():
    class ValidationError(webob_exc.WSGIHTTPException):
        code = '422'
        title = 'Validation Failed'
        explanation = 'Validation of an attribute failed.'
    def start_response(status, headers, exc_info=None):
        # check that json doesn't contain a charset
        assert ('Content-Type', 'application/json') in headers
        pass

    exc = ValidationError(detail='Attribute "xyz" is invalid.')
    resp = exc.generate_response(environ={
        'wsgi.url_scheme': 'HTTP',
        'SERVER_NAME': 'localhost',
        'SERVER_PORT': '80',
        'REQUEST_METHOD': 'PUT',
        'HTTP_ACCEPT': 'application/json',
    }, start_response=start_response)
    assert json.loads(resp[0].decode('utf-8')) == {
        "code": "422 Validation Failed",
        "title": "Validation Failed",
        "message": "Validation of an attribute failed.<br /><br />\nAttribute"
                   ' "xyz" is invalid.\n\n',
    }

def test_WSGIHTTPException_respects_accept_text_html():
    def start_response(status, headers, exc_info=None):
        for header in headers:
            if header[0] == 'Content-Type':
                assert header[1].startswith('text/html')

    exc = webob_exc.WSGIHTTPException()
    resp = exc.generate_response(environ={
        'wsgi.url_scheme': 'HTTP',
        'SERVER_NAME': 'localhost',
        'SERVER_PORT': '80',
        'REQUEST_METHOD': 'GET',
        'HTTP_ACCEPT': 'text/html',
    }, start_response=start_response)

def test_WSGIHTTPException_respects_accept_text_plain():
    def start_response(status, headers, exc_info=None):
        for header in headers:
            if header[0] == 'Content-Type':
                assert header[1].startswith('text/plain')

    exc = webob_exc.WSGIHTTPException()
    resp = exc.generate_response(environ={
        'wsgi.url_scheme': 'HTTP',
        'SERVER_NAME': 'localhost',
        'SERVER_PORT': '80',
        'REQUEST_METHOD': 'GET',
        'HTTP_ACCEPT': 'text/plain',
    }, start_response=start_response)

def test_WSGIHTTPException_respects_accept_star_star():
    def start_response(status, headers, exc_info=None):
        for header in headers:
            if header[0] == 'Content-Type':
                assert header[1].startswith('text/html')

    exc = webob_exc.WSGIHTTPException()
    resp = exc.generate_response(environ={
        'wsgi.url_scheme': 'HTTP',
        'SERVER_NAME': 'localhost',
        'SERVER_PORT': '80',
        'REQUEST_METHOD': 'GET',
        'HTTP_ACCEPT': '*/*',
    }, start_response=start_response)

def test_WSGIHTTPException_allows_custom_json_formatter():
    def json_formatter(body, status, title, environ):
        return {"fake": True}
    class ValidationError(webob_exc.WSGIHTTPException):
        code = '422'
        title = 'Validation Failed'
        explanation = 'Validation of an attribute failed.'

    exc = ValidationError(detail='Attribute "xyz" is invalid.',
                          json_formatter=json_formatter)
    body = exc.json_body({})
    assert json.loads(body) == {"fake": True}

def test_WSGIHTTPException_generate_response():
    def start_response(status, headers, exc_info=None):
        assert ('Content-Type', 'text/html; charset=UTF-8') in headers
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'PUT',
       'HTTP_ACCEPT': 'text/html'
    }
    excep = webob_exc.WSGIHTTPException()
    assert excep(environ, start_response) == [
        b'<html>\n'
        b' <head>\n'
        b'  <title>500 Internal Server Error</title>\n'
        b' </head>\n'
        b' <body>\n'
        b'  <h1>500 Internal Server Error</h1>\n'
        b'  <br /><br />\n'
        b'\n'
        b'\n\n'
        b' </body>\n'
        b'</html>']

def test_WSGIHTTPException_call_w_body():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'PUT'
    }
    excep = webob_exc.WSGIHTTPException()
    excep.body = b'test'
    assert  excep(environ,start_response) == [b'test'] 


def test_WSGIHTTPException_wsgi_response():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD'
    }
    excep = webob_exc.WSGIHTTPException()
    assert  excep.wsgi_response(environ,start_response) == [] 

def test_WSGIHTTPException_exception_newstyle():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD'
    }
    excep = webob_exc.WSGIHTTPException()
    webob_exc.newstyle_exceptions = True
    assert  excep(environ,start_response) == [] 

def test_WSGIHTTPException_exception_no_newstyle():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD'
    }
    excep = webob_exc.WSGIHTTPException()
    webob_exc.newstyle_exceptions = False
    assert  excep(environ,start_response) == [] 

def test_HTTPOk_head_of_proxied_head():
    # first set up a response to a HEAD request
    HELLO_WORLD = "Hi!\n"
    CONTENT_TYPE = "application/hello"
    def head_app(environ, start_response):
        """An application object that understands HEAD"""
        status = '200 OK'
        response_headers = [('Content-Type', CONTENT_TYPE),
                            ('Content-Length', len(HELLO_WORLD))]
        start_response(status, response_headers)

        if environ['REQUEST_METHOD'] == 'HEAD':
            return []
        else:
            return [HELLO_WORLD]

    def verify_response(resp, description):
        assert resp.content_type == CONTENT_TYPE, description
        assert resp.content_length == len(HELLO_WORLD), description
        assert resp.body == b'', description

    req = Request.blank('/', method='HEAD')
    resp1 = req.get_response(head_app)
    verify_response(resp1, "first response")

    # Copy the response like a proxy server would.
    # Copying an empty body has set content_length
    # so copy the headers only afterwards.
    resp2 = webob_exc.status_map[resp1.status_int](request=req)
    resp2.body = resp1.body
    resp2.headerlist = resp1.headerlist
    verify_response(resp2, "copied response")

    # evaluate it again
    resp3 = req.get_response(resp2)
    verify_response(resp3, "evaluated copy")

def test_HTTPMove():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD',
       'PATH_INFO': '/',
    }
    m = webob_exc._HTTPMove()
    assert  m( environ, start_response ) == [] 

def test_HTTPMove_location_not_none():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD',
       'PATH_INFO': '/',
    }
    m = webob_exc._HTTPMove(location='http://example.com')
    assert  m( environ, start_response ) == [] 

def test_HTTPMove_location_newlines():
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD',
       'PATH_INFO': '/',
    }
    with pytest.raises(ValueError):
        webob_exc._HTTPMove(location='http://example.com\r\nX-Test: false')

def test_HTTPMove_add_slash_and_location():
    def start_response(status, headers, exc_info=None):
        pass
    with pytest.raises(TypeError):
        webob_exc._HTTPMove(location='http://example.com', add_slash=True)

def test_HTTPMove_call_add_slash():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD',
       'PATH_INFO': '/',
    }
    m = webob_exc._HTTPMove()
    m.add_slash = True
    assert  m( environ, start_response ) == [] 

def test_HTTPMove_call_query_string():
    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'HEAD'
    }
    m = webob_exc._HTTPMove()
    m.add_slash = True
    environ[ 'QUERY_STRING' ] = 'querystring'
    environ['PATH_INFO'] = '/'
    assert  m( environ, start_response ) == [] 

def test_HTTPFound_unused_environ_variable():
    class Crashy(object):
        def __str__(self):
            raise Exception('I crashed!')

    def start_response(status, headers, exc_info=None):
        pass
    environ = {
       'wsgi.url_scheme': 'HTTP',
       'SERVER_NAME': 'localhost',
       'SERVER_PORT': '80',
       'REQUEST_METHOD': 'GET',
       'PATH_INFO': '/',
       'HTTP_ACCEPT': 'text/html',
       'crashy': Crashy()
    }

    m = webob_exc._HTTPMove(location='http://www.example.com')
    assert m(environ, start_response) == [
        b'<html>\n'
        b' <head>\n'
        b'  <title>500 Internal Server Error</title>\n'
        b' </head>\n'
        b' <body>\n'
        b'  <h1>500 Internal Server Error</h1>\n'
        b'  The resource has been moved to '
        b'<a href="http://www.example.com">'
        b'http://www.example.com</a>;\n'
        b'you should be redirected automatically.\n'
        b'\n\n'
        b' </body>\n'
        b'</html>']

def test_HTTPExceptionMiddleware_ok():
    def app(environ, start_response):
        return '123'
    application = app
    m = webob_exc.HTTPExceptionMiddleware(application)
    environ = {}
    start_response = None
    res = m(environ, start_response)
    assert res == '123'

def test_HTTPExceptionMiddleware_exception():
    def wsgi_response(environ, start_response):
        return '123'
    def app(environ, start_response):
        raise webob_exc.HTTPException(None, wsgi_response)
    application = app
    m = webob_exc.HTTPExceptionMiddleware(application)
    environ = {}
    start_response = None
    res = m(environ, start_response)
    assert res == '123'

def test_HTTPExceptionMiddleware_exception_exc_info_none():
    class DummySys:
        def exc_info(self):
            return None
    def wsgi_response(environ, start_response):
        return start_response('200 OK', [], exc_info=None)
    def app(environ, start_response):
        raise webob_exc.HTTPException(None, wsgi_response)
    application = app
    m = webob_exc.HTTPExceptionMiddleware(application)
    environ = {}
    def start_response(status, headers, exc_info):
        pass
    try:
        old_sys = webob_exc.sys
        sys = DummySys()
        res = m(environ, start_response)
        assert res is None
    finally:
        webob_exc.sys = old_sys

def test_status_map_is_deterministic():
    for code, cls in (
        (200, webob_exc.HTTPOk),
        (201, webob_exc.HTTPCreated),
        (202, webob_exc.HTTPAccepted),
        (203, webob_exc.HTTPNonAuthoritativeInformation),
        (204, webob_exc.HTTPNoContent),
        (205, webob_exc.HTTPResetContent),
        (206, webob_exc.HTTPPartialContent),
        (300, webob_exc.HTTPMultipleChoices),
        (301, webob_exc.HTTPMovedPermanently),
        (302, webob_exc.HTTPFound),
        (303, webob_exc.HTTPSeeOther),
        (304, webob_exc.HTTPNotModified),
        (305, webob_exc.HTTPUseProxy),
        (307, webob_exc.HTTPTemporaryRedirect),
        (308, webob_exc.HTTPPermanentRedirect),
        (400, webob_exc.HTTPBadRequest),
        (401, webob_exc.HTTPUnauthorized),
        (402, webob_exc.HTTPPaymentRequired),
        (403, webob_exc.HTTPForbidden),
        (404, webob_exc.HTTPNotFound),
        (405, webob_exc.HTTPMethodNotAllowed),
        (406, webob_exc.HTTPNotAcceptable),
        (407, webob_exc.HTTPProxyAuthenticationRequired),
        (408, webob_exc.HTTPRequestTimeout),
        (409, webob_exc.HTTPConflict),
        (410, webob_exc.HTTPGone),
        (411, webob_exc.HTTPLengthRequired),
        (412, webob_exc.HTTPPreconditionFailed),
        (413, webob_exc.HTTPRequestEntityTooLarge),
        (414, webob_exc.HTTPRequestURITooLong),
        (415, webob_exc.HTTPUnsupportedMediaType),
        (416, webob_exc.HTTPRequestRangeNotSatisfiable),
        (417, webob_exc.HTTPExpectationFailed),
        (422, webob_exc.HTTPUnprocessableEntity),
        (423, webob_exc.HTTPLocked),
        (424, webob_exc.HTTPFailedDependency),
        (428, webob_exc.HTTPPreconditionRequired),
        (429, webob_exc.HTTPTooManyRequests),
        (431, webob_exc.HTTPRequestHeaderFieldsTooLarge),
        (451, webob_exc.HTTPUnavailableForLegalReasons),
        (500, webob_exc.HTTPInternalServerError),
        (501, webob_exc.HTTPNotImplemented),
        (502, webob_exc.HTTPBadGateway),
        (503, webob_exc.HTTPServiceUnavailable),
        (504, webob_exc.HTTPGatewayTimeout),
        (505, webob_exc.HTTPVersionNotSupported),
        (511, webob_exc.HTTPNetworkAuthenticationRequired),
    ):
        assert webob_exc.status_map[code] == cls



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
try:
    with open(os.path.join(here, 'README.rst')) as f:
        README = f.read()
    with open(os.path.join(here, 'CHANGES.txt')) as f:
        CHANGES = f.read()
except IOError:
    README = CHANGES = ''

testing_extras = [
    'pytest >= 3.1.0',  # >= 3.1.0 so we can use pytest.param
    'coverage',
    'pytest-cov',
    'pytest-xdist',
    ]

docs_extras = [
    'Sphinx >= 1.7.5',
    'pylons-sphinx-themes',
    'setuptools'
    ]

setup(
    name='WebOb',
    version='1.8.10',
    description="WSGI request and response object",
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        "Development Status :: 6 - Mature",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    keywords='wsgi request web http',
    author='Ian Bicking',
    author_email='ianb@colorstudy.com',
    maintainer='Pylons Project',
    url='http://webob.org/',
    license='MIT',
    packages=find_packages('src', exclude=['tests']),
    package_dir={'': 'src'},
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*',
    install_requires=[
        "legacy-cgi>=2.6; python_version>='3.13'",
    ],
    zip_safe=True,
    extras_require={
        'testing': testing_extras,
        'docs': docs_extras,
        },
)



import unittest
from io import BytesIO

import pytest
import sys

from webob.compat import text_type

class text_Tests(unittest.TestCase):
    def _callFUT(self, *arg, **kw):
        from webob.compat import text_
        return text_(*arg, **kw)

    def test_binary(self):
        result = self._callFUT(b'123')
        self.assertTrue(isinstance(result, text_type))
        self.assertEqual(result, text_type(b'123', 'ascii'))

    def test_binary_alternate_decoding(self):
        result = self._callFUT(b'La Pe\xc3\xb1a', 'utf-8')
        self.assertTrue(isinstance(result, text_type))
        self.assertEqual(result, text_type(b'La Pe\xc3\xb1a', 'utf-8'))

    def test_binary_decoding_error(self):
        self.assertRaises(UnicodeDecodeError, self._callFUT, b'\xff', 'utf-8')

    def test_text(self):
        result = self._callFUT(text_type(b'123', 'ascii'))
        self.assertTrue(isinstance(result, text_type))
        self.assertEqual(result, text_type(b'123', 'ascii'))

class bytes_Tests(unittest.TestCase):
    def _callFUT(self, *arg, **kw):
        from webob.compat import bytes_
        return bytes_(*arg, **kw)

    def test_binary(self):
        result = self._callFUT(b'123')
        self.assertTrue(isinstance(result, bytes))
        self.assertEqual(result, b'123')

    def test_text(self):
        val = text_type(b'123', 'ascii')
        result = self._callFUT(val)
        self.assertTrue(isinstance(result, bytes))
        self.assertEqual(result, b'123')

    def test_text_alternate_encoding(self):
        val = text_type(b'La Pe\xc3\xb1a', 'utf-8')
        result = self._callFUT(val, 'utf-8')
        self.assertTrue(isinstance(result, bytes))
        self.assertEqual(result, b'La Pe\xc3\xb1a')

class Test_cgi_FieldStorage_Py3_tests(object):

    def test_fieldstorage_not_multipart(self):
        from webob.compat import cgi_FieldStorage

        POSTDATA = b'{"name": "Bert"}'

        env = {
            'REQUEST_METHOD': 'POST',
            'CONTENT_TYPE': 'text/plain',
            'CONTENT_LENGTH': str(len(POSTDATA))
        }
        fp = BytesIO(POSTDATA)
        fs = cgi_FieldStorage(fp, environ=env)
        assert fs.list is None
        assert fs.value == b'{"name": "Bert"}'

    @pytest.mark.skipif(
        sys.version_info < (3, 0),
        reason="FieldStorage on Python 2.7 is broken, see "
               "https://github.com/Pylons/webob/issues/293"
    )
    def test_fieldstorage_part_content_length(self):
        from webob.compat import cgi_FieldStorage
        BOUNDARY = "JfISa01"
        POSTDATA = """--JfISa01
Content-Disposition: form-data; name="submit-name"
Content-Length: 5

Larry
--JfISa01"""
        env = {
            'REQUEST_METHOD': 'POST',
            'CONTENT_TYPE': 'multipart/form-data; boundary={}'.format(BOUNDARY),
            'CONTENT_LENGTH': str(len(POSTDATA))}
        fp = BytesIO(POSTDATA.encode('latin-1'))
        fs = cgi_FieldStorage(fp, environ=env)
        assert len(fs.list) == 1
        assert fs.list[0].name == 'submit-name'
        assert fs.list[0].value == 'Larry'
    
    def test_my_fieldstorage_part_content_length(self):
        from webob.compat import cgi_FieldStorage
        BOUNDARY = "4ddfd368-cb07-4b9e-b003-876010298a6c"
        POSTDATA = """--4ddfd368-cb07-4b9e-b003-876010298a6c
Content-Disposition: form-data; name="object"; filename="file.txt"
Content-Type: text/plain
Content-Length: 5
Content-Transfer-Encoding: 7bit

ADMIN
--4ddfd368-cb07-4b9e-b003-876010298a6c
Content-Disposition: form-data; name="sign_date"
Content-Type: application/json; charset=UTF-8
Content-Length: 22
Content-Transfer-Encoding: 7bit

"2016-11-23T12:22:41Z"
--4ddfd368-cb07-4b9e-b003-876010298a6c
Content-Disposition: form-data; name="staffId"
Content-Type: text/plain; charset=UTF-8
Content-Length: 5
Content-Transfer-Encoding: 7bit

ADMIN
--4ddfd368-cb07-4b9e-b003-876010298a6c--"""
        env = {
            'REQUEST_METHOD': 'POST',
            'CONTENT_TYPE': 'multipart/form-data; boundary={}'.format(BOUNDARY),
            'CONTENT_LENGTH': str(len(POSTDATA))}
        fp = BytesIO(POSTDATA.encode('latin-1'))
        fs = cgi_FieldStorage(fp, environ=env)
        assert len(fs.list) == 3
        expect = [{'name':'object', 'filename':'file.txt', 'value':b'ADMIN'},
                  {'name':'sign_date', 'filename':None, 'value':'"2016-11-23T12:22:41Z"'},
                  {'name':'staffId', 'filename':None, 'value':'ADMIN'}]
        for x in range(len(fs.list)):
            for k, exp in expect[x].items():
                got = getattr(fs.list[x], k)
                assert got == exp

    def test_fieldstorage_multipart_leading_whitespace(self):
        from webob.compat import cgi_FieldStorage

        BOUNDARY = "---------------------------721837373350705526688164684"
        POSTDATA = """-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="id"

1234
-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="title"


-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="file"; filename="test.txt"
Content-Type: text/plain

Testing 123.

-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="submit"

 Add\x20
-----------------------------721837373350705526688164684--
"""

        env = {
            'REQUEST_METHOD': 'POST',
            'CONTENT_TYPE': 'multipart/form-data; boundary={}'.format(BOUNDARY),
            'CONTENT_LENGTH': '560'}
        # Add some leading whitespace to our post data that will cause the
        # first line to not be the innerboundary.
        fp = BytesIO(b"\r\n" + POSTDATA.encode('latin-1'))
        fs = cgi_FieldStorage(fp, environ=env)
        assert len(fs.list) == 4
        expect = [{'name':'id', 'filename':None, 'value':'1234'},
                  {'name':'title', 'filename':None, 'value':''},
                  {'name':'file', 'filename':'test.txt', 'value':b'Testing 123.\n'},
                  {'name':'submit', 'filename':None, 'value':' Add '}]
        for x in range(len(fs.list)):
            for k, exp in expect[x].items():
                got = getattr(fs.list[x], k)
                assert got == exp



import os
import urllib
import time
import re
from cPickle import load, dump
from webob import Request, Response, html_escape
from webob import exc

class Commenter(object):

    def __init__(self, app, storage_dir):
        self.app = app
        self.storage_dir = storage_dir
        if not os.path.exists(storage_dir):
            os.makedirs(storage_dir)

    def __call__(self, environ, start_response):
        req = Request(environ)
        if req.path_info_peek() == '.comments':
            return self.process_comment(req)(environ, start_response)
        # This is the base path of *this* middleware:
        base_url = req.application_url
        resp = req.get_response(self.app)
        if resp.content_type != 'text/html' or resp.status_code != 200:
            # Not an HTML response, we don't want to
            # do anything to it
            return resp(environ, start_response)
        # Make sure the content isn't gzipped:
        resp.decode_content()
        comments = self.get_data(req.url)
        body = resp.body
        body = self.add_to_end(body, self.format_comments(comments))
        body = self.add_to_end(body, self.submit_form(base_url, req))
        resp.body = body
        return resp(environ, start_response)

    def get_data(self, url):
        # Double-quoting makes the filename safe
        filename = self.url_filename(url)
        if not os.path.exists(filename):
            return []
        else:
            f = open(filename, 'rb')
            data = load(f)
            f.close()
            return data

    def save_data(self, url, data):
        filename = self.url_filename(url)
        f = open(filename, 'wb')
        dump(data, f)
        f.close()

    def url_filename(self, url):
        return os.path.join(self.storage_dir, urllib.quote(url, ''))

    _end_body_re = re.compile(r'</body.*?>', re.I|re.S)

    def add_to_end(self, html, extra_html):
        """
        Adds extra_html to the end of the html page (before </body>)
        """
        match = self._end_body_re.search(html)
        if not match:
            return html + extra_html
        else:
            return html[:match.start()] + extra_html + html[match.start():]

    def format_comments(self, comments):
        if not comments:
            return ''
        text = []
        text.append('<hr>')
        text.append('<h2><a name="comment-area"></a>Comments (%s):</h2>' % len(comments))
        for comment in comments:
            text.append('<h3><a href="%s">%s</a> at %s:</h3>' % (
                html_escape(comment['homepage']), html_escape(comment['name']),
                time.strftime('%c', comment['time'])))
            # Susceptible to XSS attacks!:
            text.append(comment['comments'])
        return ''.join(text)

    def submit_form(self, base_path, req):
        return '''<h2>Leave a comment:</h2>
        <form action="%s/.comments" method="POST">
         <input type="hidden" name="url" value="%s">
         <table width="100%%">
          <tr><td>Name:</td>
              <td><input type="text" name="name" style="width: 100%%"></td></tr>
          <tr><td>URL:</td>
              <td><input type="text" name="homepage" style="width: 100%%"></td></tr>
         </table>
         Comments:<br>
         <textarea name="comments" rows=10 style="width: 100%%"></textarea><br>
         <input type="submit" value="Submit comment">
        </form>
        ''' % (base_path, html_escape(req.url))

    def process_comment(self, req):
        try:
            url = req.params['url']
            name = req.params['name']
            homepage = req.params['homepage']
            comments = req.params['comments']
        except KeyError, e:
            resp = exc.HTTPBadRequest('Missing parameter: %s' % e)
            return resp
        data = self.get_data(url)
        data.append(dict(
            name=name,
            homepage=homepage,
            comments=comments,
            time=time.gmtime()))
        self.save_data(url, data)
        resp = exc.HTTPSeeOther(location=url+'#comment-area')
        return resp

if __name__ == '__main__':
    import optparse
    parser = optparse.OptionParser(
        usage='%prog --port=PORT BASE_DIRECTORY'
        )
    parser.add_option(
        '-p', '--port',
        default='8080',
        dest='port',
        type='int',
        help='Port to serve on (default 8080)')
    parser.add_option(
        '--comment-data',
        default='./comments',
        dest='comment_data',
        help='Place to put comment data into (default ./comments/)')
    options, args = parser.parse_args()
    if not args:
        parser.error('You must give a BASE_DIRECTORY')
    base_dir = args[0]
    from paste.urlparser import StaticURLParser
    app = StaticURLParser(base_dir)
    app = Commenter(app, options.comment_data)
    from wsgiref.simple_server import make_server
    httpd = make_server('localhost', options.port, app)
    print 'Serving on http://localhost:%s' % options.port
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        print '^C'
